import os
import sys
import struct

ARGV = sys.argv

def number(v):
    if type(v) is str and v[0:2] == '0x':
        v = int(v[2:],16)
    return float(v)

def istype(v,t):
    if t == 'string': return isinstance(v,str)
    elif t == 'list': return (isinstance(v,list) or isinstance(v,tuple))
    elif t == 'dict': return isinstance(v,dict)
    elif t == 'number': return (isinstance(v,float) or isinstance(v,int))
    raise '?'

def fpack(v):
    return struct.pack('f',v)

def system(cmd):
    return os.system(cmd)

def load(fname):
    f = open(fname,'r')
    r = f.read()
    f.close()
    return r

def save(fname,v):
    f = open(fname,'w')
    f.write(v)
    f.close()

def do_cmd(cmd):
    print cmd
    r = os.system(cmd)
    if r: sys.exit(r)

def build_tinypy(opt=False):
    out = []
    for mod in ['pylang','dumbparse','dump2vm','py2c']:
        out.append("""unsigned char tp_%s[] = {"""%mod)
        fname = mod+".tpc"
        data = open(fname,'rb').read()
        cols = 16
        for n in xrange(0,len(data),cols):
            out.append(",".join([str(ord(v)) for v in data[n:n+cols]])+',')
        out.append("""};""")
        do_cmd("rm -f ./"+fname)
    out.append("")
    f = open('tinypy_bc.c','wb')
    f.write('\n'.join(out))
    f.close()
    do_cmd("rm -f ./tinypy")
    if not opt:
        do_cmd("gcc -g tinypy.c `sdl-config --cflags --libs` -lm -lgc -o tinypy")
    else:
        do_cmd("gcc -O3 tinypy.c `sdl-config --cflags --libs` -lm -lgc -o tinypy")

def bootstrap():
    do_cmd('python tests.py')
    do_cmd('python py2c.py tests.py tests.tpc')
    do_cmd('python py2c.py pylang.py pylang.tpc')
    do_cmd('python py2c.py dumbparse.py dumbparse.tpc')
    do_cmd('python py2c.py dump2vm.py dump2vm.tpc')
    do_cmd('python py2c.py py2c.py py2c.tpc')
    do_cmd('./vm tests.tpc')
    do_cmd('./vm py2c.tpc tests.py tests.tpc')
    do_cmd('./vm py2c.tpc pylang.py pylang.tpc')
    do_cmd('./vm py2c.tpc dumbparse.py dumbparse.tpc')
    do_cmd('./vm py2c.tpc dump2vm.py dump2vm.tpc')
    do_cmd('./vm py2c.tpc py2c.py py2c.tpc')
    build_tinypy()
    #second pass - builts optimized binaries and stuff
    do_cmd('./tinypy tests.py')
    do_cmd('./tinypy py2c.py tests.py tests.tpc -nopos')
    do_cmd('./tinypy py2c.py pylang.py pylang.tpc -nopos')
    do_cmd('./tinypy py2c.py dumbparse.py dumbparse.tpc -nopos')
    do_cmd('./tinypy py2c.py dump2vm.py dump2vm.tpc -nopos')
    do_cmd('./tinypy py2c.py py2c.py py2c.tpc -nopos')
    build_tinypy(True)
    do_cmd('./tinypy tests.py')

if __name__ == '__main__':
    bootstrap()
